const {
  app,
  BrowserWindow,
  Menu,
  dialog,
  shell
} = require('electron')
const url = require("url");
const path = require("path");
const fs = require("fs");

let appWindow
let downloadWindow
const systemManualFolder = "./resources/app/src/assets/";

function initWindow() {
  appWindow = new BrowserWindow({
    width: 1280,
    height: 1024,
    icon: path.join(__dirname, 'assets/256x256-Icon-mit-Buch.png'),
    webPreferences: {
      contextIsolation: true,
      nodeIntegration: true,
      enableRemoteModule: false
    }
   
  })

  const template = [
    {
      Label: 'window',
      role: 'window',
      submenu: [
        {
          label: 'Back',
          accelerator: 'Backspace',
          click: (item, window) => {
            if (window) {
              window.webContents.goBack()
            }
          }
        },
        {
          label: 'Forward',
          accelerator: 'Shift+Backspace',
          click: (item, window) => {
            if (window) {
              window.webContents.goForward()
            }
          }
        }
/*        ,
        {
          label: 'Upload System Manual',
          click: (item, window) => {
            dialog.showOpenDialog(appWindow, {
              title: 'Upload system manual',
              buttonLabel: 'Upload',
              properties: ['openFile', 'multiSelections'],
              filters: [
                { name: 'SystemManuals', extensions: ['pdf'] }
              ],
              message: 'Select the system manuals you want to be included into the SDK documentation.'
            }).then(result => {
              if(!result.canceled) {
                if (fs.existsSync(systemManualFolder)) {
                  result.filePaths.forEach(filepath => {
                    const fileName = filepath.replace(/\\/g, "/").split("/").pop();
                    const newPath = systemManualFolder + '/' + fileName;
                    fs.copyFile(filepath, newPath, (err) => {
                      if (err) throw err;
                      console.log('source.txt was copied to destination.txt');
                    });
                  });
                }
              }
            }).catch(err => {
              console.log(err)
            })
            
          }
        },
        {
          label: 'Download System Manual',
          enabled: false,
          click: (item, window) => {
            // initDownloadSystemManuals()
          }
        }
*/        
      ]
    }, {
      label: 'View',
      submenu: [
         {
            role: 'resetzoom'
         },
         {
            role: 'zoomin'
         },
         {
            role: 'zoomout'
         },
         {
            type: 'separator'
         },
         {
            role: 'togglefullscreen'
         }
      ]
   }
  ]
  const menu = Menu.buildFromTemplate (template)
  Menu.setApplicationMenu (menu)  

  appWindow.setMenu(menu);
  appWindow.maximize();

  appWindow.webContents.on('will-navigate', function (e, url) {
    e.preventDefault();
    shell.openExternal(url);
  });

  // Electron Build Path
  appWindow.loadURL(
    url.format({
      pathname: path.join(__dirname, `./index.html`),
      protocol: "file:",
      slashes: true
    })
  );

  appWindow.on('app-command', (e, cmd) => {
    // Navigate the window back when the user hits their mouse back button
    if (cmd === 'browser-backward' && appWindow.webContents.canGoBack()) {
      appWindow.webContents.goBack()
    }
  })

  appWindow.webContents.on('new-window', function(e, url) {
    e.preventDefault();
    require('electron').shell.openExternal(url);
  });

  appWindow.on('closed', function () {
    appWindow = null
  })
}

app.on('ready', initWindow)

// Close when all windows are closed.
app.on('window-all-closed', function () {

  // On macOS specific close process
  if (process.platform !== 'darwin') {
    app.quit()
  }
})

app.on('activate', function () {
  if (win === null) {
    initWindow()
  }
})

/*
function initDownloadSystemManuals() {
  downloadWindow = new BrowserWindow({
    width: 1280,
    height: 800,
    webPreferences: {
      contextIsolation: true,
      nodeIntegration: false,
      enableRemoteModule: false
    }
  })

  downloadWindow.loadURL('https://www.feig.de/en/downloads-support.html')

  downloadWindow.webContents.session.on('will-download', (event, item, webContents) => {
      const url = webContents.getURL()
      if (url.startsWith('https://www.feig.de/')) {
        const file = item.getFilename()
        if( file.startsWith("H") && file.endsWith(".pdf")) {
         
          // Set the save path, making Electron not to prompt a save dialog.
          item.setSavePath('./resources/app/src/assets/' + file)

          item.on('updated', (event, state) => {
            if (state === 'interrupted') {
              console.log('Download is interrupted but can be resumed')
            } else if (state === 'progressing') {
              if (item.isPaused()) {
                console.log('Download is paused')
              } else {
                console.log(`Received bytes: ${item.getReceivedBytes()}`)
              }
            }
          })
          item.once('done', (event, state) => {
            if (state === 'completed') {
              dialog.showMessageBox({
                type: 'info',
                title: 'Download completed',
                buttons: ['Close'],
                cancelId: 0,
                message: `${item.getFilename()} download completed`,
                detail: `The system manual ${item.getFilename()} has been downloaded and is now available in the SDK documentation.`
              })
            } else {
              dialog.showMessageBox({
                type: 'error',
                title: 'Download failed',
                buttons: ['Close'],
                cancelId: 0,
                message: `The system manual ${item.getFilename()} could not be downloaded.`
              })            }
          })
        } else {
          item.cancel()
          dialog.showMessageBox({
            type: 'warning',
            title: 'File not supported',
            buttons: ['Close'],
            cancelId: 0,
            message: `File not supported by SDK Documentation Download`,
            detail: `Only system manuals are supported by the SDK documentation tool. Please use a browser to download ${item.getFilename()} from the download area.` 
          })   
        }
      }
    })
}
*/